# A plugin for adding "A-Form" functionality.
# Copyright (c) 2008 ARK-Web Co.,Ltd.

package MT::AForm;

use strict;

use MT::Object;
@MT::AForm::ISA = qw(MT::Object);
__PACKAGE__->install_properties({
	column_defs => {
		'id' => 'integer not null auto_increment',
		'title' => 'string(255) not null',
		'status' => 'string(20) not null',
		'thanks_url' => 'string(255)',
		'start_at' => 'datetime',
		'end_at' => 'datetime',
		'mail_to' => 'string(100)',
		'mail_from' => 'string(100)',
		'mail_cc' => 'string(100)',
		'mail_bcc' => 'string(100)',
		'mail_subject' => 'string(255)',
		'mail_header' => 'text',
		'mail_footer' => 'text',
		'is_replyed_to_customer' => 'boolean not null',
		'action_url' => 'string(255)',
		'data_id' => 'integer',
		'data_id_offset' => 'integer',
		'receive_limit' => 'integer',
		'msg_before_start' => 'text',
		'msg_limit_over' => 'text',
		'msg_after_end' => 'text',
		'auto_status' => 'integer',
		'check_immediate' => 'integer',
	},
	indexes => {
		'id' => 1,
                'title' => 1,
                'status' => 1,
                'start_at' => 1,
                'end_at' => 1,
                'auto_status' => 1,
	},
        defaults => {
                'is_replyed_to_customer' => 0,
                'data_id' => 0,
                'data_id_offset' => 0,
		'check_immediate' => 1,
        },
	audit => 1,
	datasource => 'aform',
	primary_key => 'id'
});

sub next {
  my $aform = shift;
  my @next_aform = MT::App->model('aform')->load( 
                     { id => [ $aform->id, undef ] },
                     { 
                       sort => 'id',
                       direction => 'ascend',
                       range => { id => 1},
                       limit => 1,
                     }
                   );
  return @next_aform ? $next_aform[0] : "";
}

sub previous {
  my $aform = shift;
  my @prev_aform = MT::App->model('aform')->load( 
                     { id => [ undef, $aform->id ] },
                     { 
                       sort => 'id',
                       direction => 'descend',
                       range => { id => 1},
                       limit => 1,
                     }
                   );
  return @prev_aform ? $prev_aform[0] : "";
}

sub set_status {
  my $aform = shift;
  my $status = shift;

  if( $status ne $aform->status ){
    $aform->set_values({status => $status});
    $aform->save;
  }
}

sub published {
  my $aform = shift;

  $aform->set_status(2);
}

sub unpublished {
  my $aform = shift;

  $aform->set_status(0);
}

sub set_values {
  my $obj = shift;
  my ($values) = @_;
  for my $col (keys %$values) {
    if( $col eq 'start_at' || $col eq 'end_at' ){
      if( $$values{$col} eq '' ){
        $$values{$col} = '1970-01-01 00:00:00';
      }
    }
  }
  return $obj->SUPER::set_values($values);
}

sub start_at {
  my $obj = shift;
  if (@_) {
    return $obj->SUPER::start_at(@_);
  }else{
    return $obj->_get_ts('start_at');
  }
}

sub end_at {
  my $obj = shift;
  if (@_) {
    return $obj->SUPER::end_at(@_);
  }else{
    return $obj->_get_ts('end_at');
  }
}

sub _get_ts {
  my $obj = shift;
  my $col = shift;

  my %values = %{ $obj->column_values };
  my $ts = $values{$col};
  if( $ts eq '19700101000000' ){
    $ts = '00000000000000';
  }
  return $ts;
}

1;

