# A plugin for adding "A-Form" functionality.
# Copyright (c) 2008 ARK-Web Co.,Ltd.

package MT::AFormData;

use strict;

use MT::Object;
@MT::AFormData::ISA = qw(MT::Object);
__PACKAGE__->install_properties({
	column_defs => {
		'id' => 'integer not null auto_increment',
                'aform_id' => 'integer not null',
                'values' => 'text',
                'aform_url' => 'string(255) not null',
	},
	indexes => {
		'id' => 1,
                'aform_id' => 1,
                'aform_url' => 1,
	},
        defaults => {
        },
	audit => 1,
	datasource => 'aform_data',
	primary_key => 'id'
});

sub top {
  my $aform_data = shift;
  my @top_aform_data = MT::App->model('aform_data')->load(
                     { 
                       aform_id => $aform_data->aform_id, 
                     },
                     {
                       sort => 'id',
                       direction => 'ascend',
                       limit => 1,
                     }
                   );
  return @top_aform_data ? $top_aform_data[0] : "";
}

sub last {
  my $aform_data = shift;
  my @last_aform_data = MT::App->model('aform_data')->load(
                     { 
                       aform_id => $aform_data->aform_id, 
                     },
                     {
                       sort => 'id',
                       direction => 'descend',
                       limit => 1,
                     }
                   );
  return @last_aform_data ? $last_aform_data[0] : "";
}

sub next {
  my $aform_data = shift;
  my @next_aform_data = MT::App->model('aform_data')->load(
                     { 
                       aform_id => $aform_data->aform_id, 
                       id => [ $aform_data->id, undef ] 
                     },
                     {
                       sort => 'id',
                       direction => 'ascend',
                       range => { id => 1},
                       limit => 1,
                     }
                   );
  return @next_aform_data ? $next_aform_data[0] : "";
}

sub previous {
  my $aform_data = shift;
  my @prev_aform_data = MT::App->model('aform_data')->load(
                     {
                       aform_id => $aform_data->aform_id,
                       id => [ undef, $aform_data->id ]
                     },
                     {
                       sort => 'id',
                       direction => 'descend',
                       range => { id => 1},
                       limit => 1,
                     }
                   );
  return @prev_aform_data ? $prev_aform_data[0] : "";
}

sub current_pos {
  my $aform_data = shift;
  my $descend = shift;

  my $count = MT::App->model('aform_data')->count(
                     {
                       aform_id => $aform_data->aform_id,
                       id => $descend ? [$aform_data->id, undef] : [undef, $aform_data->id],
                     },
                     {
                       sort => 'id',
                       range => { id => 1 },
                     });
  return $count + 1;
}

1;

