# A plugin for adding "A-Form" functionality.
# Copyright (c) 2008 ARK-Web Co.,Ltd.

package SeleniumTestCGI;

use strict;
use MT;
use MT::App;

@SeleniumTestCGI::ISA = qw(MT::App);

sub init_request {
    my $app = shift;
    $app->SUPER::init_request(@_);
    $app->add_methods( SeleniumTestCGI => \&_SeleniumTestCGI );
    $app->{default_mode} = 'SeleniumTestCGI';
    $app->{requires_login} = 0;
    $app;
}

sub _SeleniumTestCGI {
    my $app = shift;
    my $q = $app->param;

    # check params
    my $type = $app->param('type');
    return "Invalid request: type" if( $type ne 'suite' && $type ne 'case' );
    my $aform_id = $app->param('aform_id');
    return "Invalid request: aform_id" unless $aform_id;
    my $url = $app->param('url');
    return "Invalid request: url" unless $url;

    $app->{plugin_template_path} = 'plugins/AForm/tmpl';

    my $html;
    if( $type eq 'suite' ){
        $html = &disp_suite($app, $aform_id, $url);
    }elsif( $type eq 'case' ){
        $html = &disp_case($app, $aform_id, $url);
    }
    return $html;
}

sub disp_suite {
    my( $app, $aform_id, $url ) = @_;

    my %param = (
        aform_id => $aform_id,
        url => $url,
    );
    my $html = $app->load_tmpl('selenium_test_suite.tmpl', \%param);
    return $app->build_page($html, \%param);
}

sub disp_case {
    my( $app, $aform_id, $url ) = @_;

    require AFormEngineCGI::FormMail;

    my $aform = MT::AForm->load($aform_id);
    my $fields = AFormEngineCGI::FormMail::_get_fields($app, $aform);

    # open だけのテストかどうか
    my $open_only = (!&_is_aform_active($aform) || !&_is_same_domain($app->base, $url) ) ? 1 : 0;
    # 完了ページへ飛んだことをチェックするかどうか
    my $check_thanks_url = ($aform->thanks_url ne "" && &_is_same_domain($app->base, $aform->thanks_url)) ? 1 : 0;

    my %param = (
        aform_id => $aform_id,
        url => $url,
        open_only => $open_only,
        thanks_url => $aform->thanks_url,
        check_thanks_url => $check_thanks_url,
        fields => AFormEngineCGI::FormMail::_get_fields($app, $aform),
    );
    my $html = $app->load_tmpl('selenium_test_case.tmpl', \%param);
    return $app->build_page($html, \%param);
}

sub _is_aform_active {
  my $aform = shift;

  my($sec,$min,$hour,$mday,$mon,$year) = localtime(time);
  my $now = sprintf("%04d%02d%02d%02d%02d%02d", $year+1900,$mon+1,$mday,$hour,$min,$sec);
  my $received_count = MT::AFormData->count({aform_id => $aform->id});

  my $active = 0;
  if( $aform->status eq 2 ){
    if( $aform->start_at > 0 || $aform->end_at > 0 || $aform->receive_limit > 0 ){
      if( $aform->start_at > 0 && $aform->start_at > $now ){
        $active = 0;
      }elsif( $aform->end_at > 0 && $aform->end_at < $now ){
        $active = 0;
      }elsif( $aform->receive_limit > 0 && $received_count >= $aform->receive_limit ){
        $active = 0;
      }else{
        $active = 1;
      }
    }else{
      $active = 1;
    }
  }
  return $active;
}


sub _is_same_domain {
    my( $url1, $url2 ) = @_;

    require URI;
    return( URI->new($url1)->host eq URI->new($url2)->host );
}

1;
